<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://la-studioweb.com/
 * @since      1.0.0
 *
 * @package    LaStudio_Pagespeed
 * @subpackage LaStudio_Pagespeed/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    LaStudio_Pagespeed
 * @subpackage LaStudio_Pagespeed/public
 * @author     LA-Studio <info@la-studioweb.com>
 */
class LaStudio_Pagespeed_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in LaStudio_Pagespeed_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The LaStudio_Pagespeed_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/lastudio-pagespeed-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in LaStudio_Pagespeed_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The LaStudio_Pagespeed_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/lastudio-pagespeed-public.js', array( 'jquery' ), $this->version, false );

	}

	public function get_default_attributes(){
		$attributes = [];
		$attributes[] = 'data-lastudiopagespeed-nooptimize="true"';
		return join(' ', $attributes);
	}

	public function get_extra_attributes(){
		$attributes = [];
		/* does not optimize script tag if server using cloudflare */
		if (!empty($_SERVER['HTTP_CF_CONNECTING_IP'])) {
			$attributes[] = 'data-cfasync="false"';
		}
		/* does not optimize script if LightSpeedCache is enabled */
		if (defined('LSCWP_V')) {
			$attributes[] = 'data-no-optimize="true"';
		}
		return join(' ', $attributes);
	}

	protected function get_lazyload_script_source(){
		$source = '';
		return $source;
	}

	protected function get_primary_script_source(){
		$content = file_get_contents(plugin_dir_path( __FILE__ ) . 'js/lastudio-pagespeed-public.js');

		$config = [
			'delay' => 4000,
			'version' => $this->version,
			'nativelazyoad' => true,
			'e_animation' => class_exists('Elementor\Plugin') ? true : false,
			'detected' => true
		];

		$extra = 'var LaStudioPageSpeedConfigs = ' . json_encode($config) . ';';
		$content = $extra . $content;

		$source = sprintf(
			'<script %1$s %2$s>%3$s</script>',
			$this->get_default_attributes(),
			$this->get_extra_attributes(),
			$content
		);
		return $source;
	}

	public function frontend_rewrite( $output ){

		$default_attributes = $this->get_default_attributes();
		$extra_attributes   = $this->get_extra_attributes();
		$action_attribute   = 'data-lastudiopagespeed-action="reorder"';
		$use_native_lazyload = false;

		/* Start including snippet */
		$output = preg_replace('/(<head\b[^>]*?>)/', '${1}' . $this->get_primary_script_source(), $output);
		/* End including snippet */

		/* start excluding native WP Rocket OR Autoptimize images lazyload */
		$output = preg_replace_callback(
			'/<script([^>]*?)>(\s*window\.lazyLoadOptions\s*=|\s*window\.lazySizesConfig\s*=)/i',
			function ($matches) use ( $default_attributes, &$use_native_lazyload ) {
				$use_native_lazyload = true;
				list($tag, $attrs, $content) = $matches;
				return '<script '. $default_attributes .' ' . $attrs . '>' . $content;
			},
			$output
		);
		$output = preg_replace_callback(
			'/<script[^>]*?>/i',
			function ($matches) use ( $default_attributes, &$use_native_lazyload ) {
				$use_native_lazyload = true;
				list ($tag) = $matches;
				if (preg_match('/\/(autoptimize|wp-rocket)\/(.*)(lazyload|lazysizes)\.min\.js/i', $tag)) {
					return preg_replace('/<script/', '<script ' . $default_attributes, $tag);
				}
				return $tag;
			},
			$output
		);
		/* end excluding native WP Rocket OR Autoptimize images lazyload */

		if ($use_native_lazyload == true) {
			$output = preg_replace('/(<head\b[^>]*?>)/', '${1}' . $this->get_lazyload_script_source(), $output);
		}

		$output = preg_replace_callback('/<script[^>]*?>/i', function ( $matches ) use ( $default_attributes, $extra_attributes, $action_attribute ) {

			list($tag) = $matches;
			$result = $tag;
			$more_attr = ' ' . $action_attribute;
			$more_attr .= ' ' . $extra_attributes;

			if (!preg_match('/\s+data-src=/i', $tag)
			    && !preg_match('/'.$default_attributes.'/i', $tag)
			    && !preg_match('/data-rocketlazyloadscript=/i', $tag, $matches)) {

				if (preg_match('/\s+src=/i', $tag, $matches)) {
					$result = preg_replace('/\s+src=/i', $more_attr . ' data-src=', $tag);
					if (preg_match('/\s+type=/i', $tag)) {
						$result = preg_replace('/\s+type=([\'"])text\/javascript\1/i', ' type="javascript/blocked"', $result);
					}
					else {
						$result = preg_replace('/<script/i', '<script type="javascript/blocked"', $result);
					}
				}
				else {
					if (preg_match('/\s+type=/i', $tag)) {
						$result = preg_replace('/\s+type=([\'"])text\/javascript\1/i', $more_attr . ' type="javascript/blocked"', $tag);
					}
					else {
						$result = preg_replace('/<script/i', '<script'.$more_attr.' type="javascript/blocked"', $tag);
					}
				}
			}

			/* Clean the output */
			$result = preg_replace('/\s*'.$default_attributes.'/i', '', $result);

			return $result;

		}, $output);

		// JetPack Likes workaround - lazyloading iframe so it don't window.postMessage early
		$output = preg_replace_callback(
			'/<iframe[^>]*?>/i',
			function ($matches) use ($default_attributes, $extra_attributes, $action_attribute) {
				list($tag) = $matches;
				if (preg_match('/\s+src=(["\'])https?:\/\/widgets\.wp\.com/i', $tag)) {
					$more_attr = ' ' . $action_attribute;
					$more_attr .= ' ' . $extra_attributes;
					return preg_replace('/\s+src=/i', $more_attr .' data-src=', $tag);
				}
				return $tag;
			},
			$output
		);

		return $output;
	}

}
